// Copyright (C) 2001 Kai Germaschewski
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "driver.h"
#include <capi20.h>

bool CapiTrunk::idleHandler(TrunkEvent *event)
{
	slog(SLOG_DEBUG) << "CAPI: " << __FUNCTION__ << " " 
			 << event->id << endl;

	const char *start = NULL;
	const char *data;
	const char *value;
	int argc = 0;
	char script[256];
	char *sp;
	unsigned err;

	switch(event->id) {
	case TRUNK_ENTER_STATE:
		debug->DebugState(this, "idle");
		switch(flags.trunk) {
		case TRUNK_MODE_OUTGOING:
			group->decOutgoing();
			break;
		case TRUNK_MODE_INCOMING:
			group->decIncoming();
		}
		tgi.pid = 0;
		disjoin();
		flags.trunk = TRUNK_MODE_INACTIVE;
		flags.ready = true;
		flags.offhook = false;
		endTimer();
		rings = 0;
		return true;
	case TRUNK_START_SCRIPT:
		start = event->parm.argv[0];
		++argc;
		
		if (Attach(start)) {
			while (event->parm.argv[argc]) {
				strcpy(script, event->parm.argv[argc++]);
				data = strtok_r(script, "=", &sp);
				value = strtok_r(NULL, "=", &sp);
				if(data && value)
					setConstant(data, value);
			}		
		
			endTimer();
			handler = &CapiTrunk::stepHandler;
			flags.trunk = TRUNK_MODE_OUTGOING;
			group->incOutgoing();
			return true;
		}

		slog(SLOG_ERROR) << "CAPI: " << id << ": " << start 
				 << ": cannot start" << endl;
		return true;		
	case TRUNK_CALL_DETECT:
	case TRUNK_RINGING_ON:
		_cmsg cmsg;

		rings = 0;
		if(!group->getAnswer())
			return true;
		
		CapiDriver *drv = (CapiDriver *) driver;
		
		group->incIncoming();
		endTimer();
		err = ALERT_REQ(&cmsg, drv->appl_id, drv->msg_id++,
				plci,
				(unsigned char *)"",
				(unsigned char *)"",
				(unsigned char *)"",
				(unsigned char *)"");
		if (err != CapiNoError) {
			slog(SLOG_ERROR) << "CAPI: ALERT_REQ err = "
					 << err << endl;
		}
		handler = &CapiTrunk::ringHandler;
		return true;
	}
	return false;
}

