// Copyright (C) 2001 Kai Germaschewski
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "driver.h"
#include <capi20.h>

bool CapiTrunk::sleepHandler(TrunkEvent *event)
{
	slog(SLOG_DEBUG) << "CAPI: " << __FUNCTION__ << " " 
			 << event->id << endl;

	timeout_t wakeup;
	char buffer[16];
	char *cp;
	char *value;
	char *sp;

	switch(event->id) {
	case TRUNK_ENTER_STATE:
		debug->DebugState(this, "sleep");
		Trunk::setDTMFDetect();
		wakeup = data.sleep.wakeup;
		if(data.sleep.rings) // FIXME
			if(rings >= data.sleep.rings)
				wakeup = 0;

	
		if (wakeup == 0) {
			TrunkSignal(TRUNK_SIGNAL_STEP);
			handler = &CapiTrunk::stepHandler;
			return true;
		}

		setTimer(wakeup);
		return true;
	case TRUNK_TIMER_EXPIRED:
		if (--data.sleep.loops)	{
			setTimer(data.sleep.wakeup);
			return true;
		}
		if(tgi.pid) {
			kill(tgi.pid, SIGALRM);
			tgi.pid = 0;
			if (!TrunkSignal(TRUNK_SIGNAL_TIMEOUT))
				TrunkSignal(TRUNK_SIGNAL_STEP);
		} else {
			TrunkSignal(TRUNK_SIGNAL_STEP);
		}
		handler = &CapiTrunk::stepHandler;
		return true;
	case TRUNK_DTMF_KEYUP:
		if(tgi.pid) {
			kill(tgi.pid, SIGINT);
			tgi.pid = 0;
		}
		return false;
	case TRUNK_EXIT_SHELL:
		if(!tgi.pid)
			return true;
		tgi.pid = 0;
		endTimer();
		sprintf(buffer, "exit-%d", event->parm.status);
		setSymbol(SYM_ERROR, buffer);
		if(event->parm.status)
			TrunkSignal(TRUNK_SIGNAL_ERROR);
		else
			TrunkSignal(TRUNK_SIGNAL_STEP);
		handler = &CapiTrunk::stepHandler;
		return true;	
#if 0
	case TRUNK_RINGING_OFF:
		if(rings > 1)
			stopDSP();
		if(!data.sleep.rings)
			return true;
		if(rings < data.sleep.rings)
			return true;
		endTimer();
	case TRUNK_SERVICE_LOOKUP:
		if(event->parm.lookup.seq != tgi.seq)
			return false;
		if(event->parm.lookup.result)
		{
			tgi.data = event->parm.lookup.data;
			cp = strtok_r(event->parm.lookup.data, "&", &sp);
			while(cp)
			{
				value = strchr(cp, '=');
				if(value)
				{
					*(value++) = 0;
					setSymbol(cp, getSymbolSize());
					setSymbol(cp, value);
				}
				cp = strtok_r(NULL, "&", &sp);
			}
			TrunkSignal(TRUNK_SIGNAL_STEP);
		}
		else
		{
			setSymbol(SYM_ERROR, event->parm.lookup.data);
			TrunkSignal(TRUNK_SIGNAL_ERROR);
		}
		handler = &PikaTrunk::stepHandler;
		return true;
#endif
	}
	return false;
}
