// Copyright (C) 2000 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// As a special exception to the GNU General Public License, permission is
// granted for additional uses of the text contained in its release
// of Bayonne as noted here.
//
// This exception is that permission is hereby granted to link Bayonne 
// with the Dialogic runtime libraries to produce a executable image
// without requiring Dialogic's sources to be supplied so long as each
// source file so linked contains this exclusion.
//
// This exception does not however invalidate any other reasons why
// the resulting executable file might be covered by the GNU General
// public license or invalidate the licensing requirements of any
// other component or library.
//
// This exception applies only to the code released by OST under the
// name Bayonne.  If you copy code from other releases into a copy of
// Bayonne, as the General Public License permits, the exception does not
// apply to the code that you add in this way.  To avoid misleading
// anyone as to the status of such modified files, you must delete
// this exception notice from them.
//
// If you write modifications of your own to Bayonne, it is your choice
// whether to permit this exception to apply to your modifications.
// If you do not wish that, delete this exception notice, at which
// point the terms of your modification would be covered under the GPL
// as explicitly stated in "COPYING".

#include "driver.h"

bool DialogicTrunk::dialHandler(TrunkEvent *event)
{
        int rtn;

#ifdef DIALOGIC_ISDN
	if (isdn && event->id == TRUNK_ENTER_STATE)
	{
		debug->DebugState(this, "dial");
		TimerPort::endTimer();
		setSymbol(SYM_PROGRESS, "none");
		Trunk::flags.offhook = true;
		event->id = TRUNK_OFF_HOOK;
	}
#endif

	switch(event->id)
	{
	case TRUNK_TIMER_EXPIRED:
		stopChannel(EV_ASYNC);
	case TRUNK_CPA_BUSYTONE:
		setSymbol(SYM_PROGRESS, "busy");
		if(!TrunkSignal(TRUNK_SIGNAL_BUSY))
		{
			if(data.dialxfer.exit)
				TrunkSignal(TRUNK_SIGNAL_HANGUP);
			else
				TrunkSignal(TRUNK_SIGNAL_STEP);
		}
		handler = &DialogicTrunk::stepHandler;
#ifdef DIALOGIC_ISDN
		if(isdn)
			setISDNHook(event->parm.crn, false);
#endif
		return true;		
	case TRUNK_CPA_NOANSWER:
		setSymbol(SYM_PROGRESS, "noanswer");
		if(TrunkSignal(TRUNK_SIGNAL_NOANSWER))
		{
			handler = &DialogicTrunk::stepHandler;
			return true;
		}
		event->id = TRUNK_TIMER_EXPIRED;
		return false;
	case TRUNK_CPA_NORINGBACK:
		setSymbol(SYM_PROGRESS, "noringback");
		event->id = TRUNK_TIMER_EXPIRED;
		return false;
	case TRUNK_CPA_NODIALTONE:
		setSymbol(SYM_PROGRESS, "nodialtone");
		event->id = TRUNK_TIMER_EXPIRED;
		return false;
	case TRUNK_TONE_IDLE:
		if(data.dialxfer.exit)
			TrunkSignal(TRUNK_SIGNAL_HANGUP);
		else
			TrunkSignal(TRUNK_SIGNAL_STEP);
		handler = &DialogicTrunk::stepHandler;
		return true;
	case TRUNK_ENTER_STATE:
		debug->DebugState(this, "dial");
		TimerPort::endTimer();
		setSymbol(SYM_PROGRESS, "none");
		setDTMFDetect(false);

		if(!Trunk::flags.offhook)
		{
			setHook(true);
			return true;
		}
	case TRUNK_OFF_HOOK:
		status[id] = 'd';
#ifdef	DIALOGIC_ISDN
		if(isdn)
		{
			dialISDN(data.dialxfer.digit, data.dialxfer.timeout);
			dx_dial(chdev, "", NULL, EV_ASYNC | DX_CALLP);
			return true;
		}
#endif
		if(data.dialxfer.timeout)
		{
			dx_dial(chdev, data.dialxfer.digit, NULL, EV_ASYNC | DX_CALLP);
			TimerPort::setTimer(data.dialxfer.timeout + 120 * strlen(data.dialxfer.digit));
		}
		else
			dx_dial(chdev, data.dialxfer.digit, NULL, EV_ASYNC);
	        return true;

	case TRUNK_LINE_WINK:
	        return true;
	}

	return false;
}
