// Copyright (C) 2000 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// As a special exception to the GNU General Public License, permission is
// granted for additional uses of the text contained in its release
// of Bayonne as noted here.
//
// This exception is that permission is hereby granted to link Bayonne 
// with the Dialogic runtime libraries to produce a executable image
// without requiring Dialogic's sources to be supplied so long as each
// each source file so linked contains this exclusion.
//
// This exception does not however invalidate any other reasons why
// the resulting executable file might be covered by the GNU General
// public license or invalidate the licensing requirements of any
// other component or library.
//
// This exception applies only to the code released by OST under the
// name Bayonne.  If you copy code from other releases into a copy of
// Bayonne, as the General Public License permits, the exception does not
// apply to the code that you add in this way.  To avoid misleading
// anyone as to the status of such modified files, you must delete
// this exception notice from them.
//
// If you write modifications of your own to Bayonne, it is your choice
// whether to permit this exception to apply to your modifications.
// If you do not wish that, delete this exception notice, at which
// point the terms of your modification would be covered under the GPL
// as explicitly stated in "COPYING".

#include "driver.h"

extern "C" {
	// "null" operation from our point of view
	
	long uio_seek(int fd, long offset, int wence)
	{
		return offset;
	}

	// read audio buffer

	int uio_read(int fd, char *buffer, unsigned len)
	{
		int rtn;
		DialogicTrunk *trunk = dialogicivr.ports[fd];
		if(!trunk)
			return -1;

		rtn = trunk->getBuffer(buffer, len);
		return rtn;
	}

	// write audio buffer

	int uio_write(int fd, char *buffer, unsigned len)
	{
		DialogicTrunk *trunk = dialogicivr.ports[fd];
		if(!trunk)
			return -1;

		return trunk->putBuffer(buffer, len);
	}
};

void DialogicTrunk::endPlay(void)
{
	char buffer[12];

	stopChannel(EV_ASYNC);
	sprintf(buffer, "%ld", getTransfered());
	trunk->setSymbol(SYM_PLAYED, buffer);
	URLAudio::Close();
}

void DialogicTrunk::endRecord(void)
{
	char buffer[12];
	struct stat ino;
	int trim;

	stopChannel(EV_ASYNC);
	sprintf(buffer, "%ld", getPosition());
	trunk->setSymbol(SYM_RECORDED, buffer);
	trim = tobytes(getEncoding(), atoi(trunk->getSymbol(SYM_TRIM)));
	stat(data.record.name, &ino);
	truncate(data.record.name, ino.st_size - trim);
	chown(data.record.name, keyserver.getUid(), keyserver.getGid());
	URLAudio::Close();
}
	

