// Copyright (C) 2000 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// 
#include "driver.h"

#include <pthread.h>

bool VPBTrunk::dialHandler(TrunkEvent *event)
{
	pthread_t DialThread;

	
	switch(event->id)
	{
	case TRUNK_CPA_NOANSWER:
		endTimer();
		if(!TrunkSignal(TRUNK_SIGNAL_NOANSWER))
		{
			event->id = TRUNK_TIMER_EXPIRED;
			return false;
		}
		handler = &VPBTrunk::stepHandler;
		return true;			
	case TRUNK_CPA_BUSYTONE:
		endTimer();
		if(!TrunkSignal(TRUNK_SIGNAL_BUSY))
			TrunkSignal(TRUNK_SIGNAL_STEP);
		handler = &VPBTrunk::stepHandler;
		return true;	
	case TRUNK_ENTER_STATE:
		slog(SLOG_INFO) <<__FILE__<<" line "<<__LINE__<< " received  TRUNK_ENTER_STATE"<<endl;	
		debug->DebugState(this, "flashoff");
		flags.offhook = true;
		vpb_sethook_async(handle, VPB_OFFHOOK);
		setSymbol(SYM_PROGRESS, "none");

		// DR 3/10/01 switch DTMF detection off while dialing to
		// prevent outbound digits being detected
		vpb_disable_event(handle, VPB_MDTMF);

		if(data.dialxfer.offhook < getPickupTimer())
			data.dialxfer.offhook = getPickupTimer();
		setTimer(data.dialxfer.offhook);
		slog(SLOG_INFO) <<__FILE__<<" line "<<__LINE__<< " start dial thread..."<<endl;
		thread = new VPBDialer(this, handle);
		thread->Start();
		return true;
	
	case TRUNK_SERVICE_SUCCESS:
	case TRUNK_SERVICE_FAILURE:
        case TRUNK_LINE_WINK:

		// DR 3/10/01 switch DTMF detection back on
		vpb_enable_event(handle, VPB_MDTMF);
	
		slog(SLOG_INFO) << " received TRUNK_LINE_WINK" <<endl;
		if(data.dialxfer.exit)
			TrunkSignal(TRUNK_SIGNAL_HANGUP);
		else
			TrunkSignal(TRUNK_SIGNAL_STEP);
		handler = &VPBTrunk::stepHandler;

                return true;
	}

	slog(SLOG_INFO) <<__FILE__<<" line "<<__LINE__<< " received not  TRUNK_ENTER_STATE and not TRUNK_LINE_WINK, it is="<< event->id <<endl;
	return false;
}


VPBDialer::VPBDialer(VPBTrunk *t, int h) :
Service(t, keythreads.priAudio())
{
	handle = h;
	reset = false;
	trk = t;
	trk->svcCallSuccess = true;
	trk->svcRunFinished = false;
}

VPBDialer::~VPBDialer()
{
	trk->svcCallSuccess = false;
	while(!trk->svcRunFinished);
	Terminate();
}

void VPBDialer::Run(void)
{
	TrunkEvent event;
	int status;
	VPB_CALL	cpp;
	
	setCancel(THREAD_CANCEL_DEFERRED);
	if(trk->data.dialxfer.timeout)
	{
		vpb_get_call(handle, &cpp);
		cpp.answer_timeout = trk->data.dialxfer.timeout;
		vpb_set_call(handle, &cpp);
		status = vpb_call_sync(handle, trk->data.dialxfer.digits);
	}
	else
	{
		vpb_dial_sync(handle, trk->data.dialxfer.digits);
		status = 0;
	}
	trk->svcRunFinished = true;
	event.id = (trunkevent_t)0;
	switch(status)
	{
	case 1:	// no dialtone
		trk->setSymbol(SYM_PROGRESS, "nodialtone");
		event.id = TRUNK_TIMER_EXPIRED;
		break;
	case 2:	// no ringback
		trk->setSymbol(SYM_PROGRESS, "noringback");
		event.id = TRUNK_TIMER_EXPIRED;
		break; 
	case 3:	// busy
		trk->setSymbol(SYM_PROGRESS, "busy");
		event.id = TRUNK_CPA_BUSYTONE;
		break;
	case 4:	// no answer
		trk->setSymbol(SYM_PROGRESS, "noanswer");
		event.id = TRUNK_CPA_NOANSWER;
		break;
	case 5:	// disconnect
		event.id = TRUNK_STOP_DISCONNECT;
		break;
	}
	if(trk->svcCallSuccess)
	{
		if(event.id)
			trk->postEvent(&event);
		else
			Success();	
	}
}






