// Copyright (C) 2000 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <ivrconfig.h>
#include <server.h>
#include <vpbapi.h>
#include <cc++/rtp.h>
#include <cc++/rtpext.h>
#include <pthread.h>

#define CID_BUF_SZ 8000*4  // DR - up to 10 seconds of CID buffer

class VPBTrunk;
class VPBThread;

typedef bool (VPBTrunk::*trunkhandler_t)(TrunkEvent *event);

class VPBTrunk: private TimerPort, private Trunk
{
private:
	friend class VPBDriver;
	friend class VPBThread;
	friend class VPBCid;
	friend class VPBDialer;

	static char status[16 * 4];

	int handle;
	int x;
	time_t lastring;
	trunkhandler_t handler;
	VPBThread *service;
	trunkevent_t notify;
	short cidbuf[CID_BUF_SZ]; // DR - storage for CID signal
	char cidnbr[VPB_MAX_STR]; // DS - storage for cid parsing

	VPBTrunk(VPBThread *thread, int ts);
	~VPBTrunk();

	void getName(char *buffer);
	unsigned long getIdleTime(void);
	unsigned getPickupTimer(void);

	bool scrRTP(void);
	
	bool stepHandler(TrunkEvent *event);
	bool busyHandler(TrunkEvent *event);
	bool idleHandler(TrunkEvent *event);
	bool seizeHandler(TrunkEvent *event);
	bool ringHandler(TrunkEvent *event);
	bool waitHandler(TrunkEvent *event);
	bool loadHandler(TrunkEvent *event);
	bool dialHandler(TrunkEvent *event);
	bool playHandler(TrunkEvent *event);
	bool playStopHandler(TrunkEvent *event);
	bool playwaitHandler(TrunkEvent *event);
	bool recordHandler(TrunkEvent *event);
	bool sleepHandler(TrunkEvent *event);
	bool hangupHandler(TrunkEvent *event);
	bool answerHandler(TrunkEvent *event);
	bool collectHandler(TrunkEvent *event);
	bool flashonHandler(TrunkEvent *event);
	bool flashoffHandler(TrunkEvent *event);
	bool rtpHandler(TrunkEvent *event);

	void setTimer(timeout_t timeout = 0, trunkevent_t notice = TRUNK_TIMER_EXPIRED);
	void setOffhook(void);

	bool postEvent(TrunkEvent *event);
	void Exit(void);
	void TrunkStep(trunkstep_t step);
	

 public:
	volatile bool svcCallSuccess; // DR 
	volatile bool svcRunFinished; // DR
};

/** THIS HAS TO BE RE-WRITTEN

class VPBRTP : private RTPSocket, public Service, public TimerPort
{
private:
	bool reset;
	int handle;
	payload_t payload;
	audioencoding_t encoding;
	unsigned frame;	

	void Initial(void);
	void Run(void);

public:
	VPBRTP(VPBTrunk *trunk, int h, InetAddress &bind, tpport_t port, audioencoding_t codec);
	~VPBRTP();
};

*/

class VPBDialer : private Service
{
private:
	friend class VPBTrunk;

	VPBTrunk *trk;
	int handle;
	bool reset;

	void Run(void);

	VPBDialer(VPBTrunk *trk, int h);
	~VPBDialer();
};

class VPBCid : private Service
{
private:
	friend class VPBTrunk;

	VPBTrunk *trk;
	int handle;
	bool reset;

	void Run(void);

	VPBCid(VPBTrunk *trunk, int h);
	~VPBCid();
};

class VPBRecord: private AudioFile, public Service
{
private:
	bool reset;
	size_t bufsize, samples;
	int handle;
	VPBTrunk *vpbtr; // DR

	void Initial(void);
	void Run(void);

public:
	VPBRecord(VPBTrunk *trunk, int h);
	~VPBRecord();
};

class VPBPlay : private URLAudio, public Service
{
private:
	bool reset;
	size_t bufsize, samples;
	int handle;
	VPBTrunk *vpbtr; // DR

	void Initial(void);
	void Run(void);
	char *getContinuation(void);

public:
	VPBPlay(VPBTrunk *trunk, int h);
	~VPBPlay();
};

class VPBConfig : public Keydata
{
public:
	VPBConfig();

	unsigned getOffset(void)
		{return strtol(getLast("first"), NULL, 16);};

	unsigned getCards(void)
		{return atoi(getLast("cards"));};

	inline unsigned getHookTimer(void)
		{return atoi(getLast("hooktime"));};

	size_t getStack(void);

	const char *getFirmware(void)
		{return getLast("firmware");};

	unsigned getDelay(void)
		{return atoi(getLast("delay"));};

	int getModel(void);
	
};

class VPBThread : public Mutex, public Thread
{
private:
	friend class VPBDriver;
	friend class VPBTrunk;

	static int count; 
	volatile bool active;

	int id;
	VPBTrunk *handles[4];
	void *reg;

	void Run(void);
	VPBThread();
	~VPBThread();
};

class VPBDriver : public Driver, public VPBConfig
{
private:
	VPBThread *threads;
	VPBTrunk **ports;
	int port_count, thread_count;

public:
	VPBDriver();
	~VPBDriver();

	int Start(void);
	void Stop(void);

	int getTrunkCount(void)
		{return port_count;};

	Trunk *getTrunkPort(int id);
};

extern VPBDriver vpbivr;



