// Copyright (C) 2000-2001 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "server.h"

Server *Server::first = NULL;
Audit *Audit::first = NULL;
Auditdata Audit::keys;
Translator *Translator::first = NULL;
TGI *TGI::first = NULL;
Module *Module::modFirst = NULL;
Module *Module::sesFirst = NULL;
Module *Module::cmdFirst = NULL;
Module *Module::urlFirst = NULL;
Module *Module::reqFirst = NULL;
Module *Module::netFirst = NULL;
Sync *Sync::first = NULL;

Auditdata::Auditdata()
{
	Load("/bayonne/audit");
	Load("~bayonne/audit");
}

Audit::Audit() :
Mutex()
{
	next = first;
	first = this;
}

Driver::Driver() :
aaScript()
{
	active = false;
	if(driver)
		throw(this);

	groups = NULL;
	driver = this;
	status = NULL;
}

aaImage *Driver::getImage(void)
{
	return new aaImage((aaScript *)this);
}

int Driver::getTrunkMember(TrunkGroup *group, unsigned member)
{
	int id, count = getTrunkCount();
	Trunk *trunk;

	for(id = 0; id < count; ++id)
	{
		if(groups[id] != group)
			continue;

		trunk = getTrunkPort(id);
		if(trunk->getMemberId() == member)
			return id;
	}
	return -1;
}

void Driver::getStatus(char *buffer)
{
	if(!status) {
		buffer[0] = 0;
		return;
	}
	memcpy(buffer, status, getTrunkCount());
}

void Functions::Load(FUNCTIONS *map)
{
	int count = 0;
	while(map->name)
	{
		driver->addFunction(map->name, map->function);
		++map;
		++count;
	}
	slog(SLOG_DEBUG) << "dso: loaded " << count << " functions" << endl;
}

void alog(Trunk *trunk, char *detail)
{
	Audit *au = Audit::first;

	while(au)
	{
		au->ReportAudit(trunk, detail);
		au = au->next;
	}
}

void audit(Trunk *trunk, char *detail)
{
	Audit *au = Audit::first;

	while(au)
	{
		au->ReportCDR(trunk, detail);
		au = au->next;
	}
}

Debug::Debug() :
Mutex()
{
	if(debug)
		throw this;

	debug = this;
}

Translator::Translator(const char *conf, const char *lib) :
Keydata(conf)
{
	char *etc = getenv("CONFIG_KEYDATA");
	char keypath[33];
	char cfgpath[128];
	char *cp;

	next = first;
	first = this;

	if(!etc)
		etc = ETC_PREFIX;

	strcpy(cfgpath, etc);
	strcat(cfgpath, "bayonne.d");

	if(lib)
		Load(lib);

	if(isDir(cfgpath))
		if(canAccess(cfgpath))
			scanDir(cfgpath, conf, lib);

	strcpy(keypath, conf);
	*keypath = '~';
	Load(keypath);

	if(!lib)
		return;

	cp = strrchr(lib, '/');
	strcpy(keypath, keyserver.getPhrases());
	strcat(keypath, ++cp);
	Load(keypath);
	strcpy(keypath, keyserver.getPhrases());
	strcat(keypath, lib);
	Load(keypath);
}

void Translator::scanDir(const char *dirpath, const char *lang, const char *lib)
{
	char buffer[65];
	char *name, *ext;
	Dir dir(dirpath);
	if(*lang == '/')
		lang = strrchr(lang, '/');

	if(lib)
		if(*lib == '/')
			lib = strrchr(lib, '/');

	while(NULL != (name = dir.getName()))
	{
		ext = strrchr(name, '.');
		if(!ext)
			continue;

		if(strcmp(ext, ".conf"))
			continue;

		strcpy(buffer, "/bayonne.d/");
		*ext = 0;
		strcat(buffer, name);
		strcat(buffer, lang);
		Load(buffer);
		if(!lib)
			continue;
		strcpy(buffer, "/bayonne.d/");
		strcat(buffer, name);
		strcat(buffer, "lib");
		Load(buffer);
	}
}

char *Translator::getPlayBuffer(Trunk *trunk)
{
	char *pbuf;
	
	pbuf = trunk->data.play.list;
	trunk->data.play.name = pbuf;
	trunk->data.play.limit = trunk->data.play.offset = 0;
	*pbuf = 0;
	return pbuf;
}

Translator *getTranslator(char *name)
{
	Translator *trans = Translator::first;

	while(trans)
	{
		if(!stricmp(name, trans->getName()))
			return trans;
		trans = trans->next;
	}
	return NULL;
}

Sync::Sync()
{
	next = first;
	first = this;
	time(&runtime);
}

Server::Server(int pri) :
#ifdef	COMMON_OST_NAMESPACE
Thread(pri, keythreads.getStack())
#else
Thread(NULL, pri, keythreads.getStack())
#endif
{
	next = first;
	first = this;
}

void startServers(void)
{
	Server *server = Server::first;

	while(server)
	{
		server->Start();
		server = server->next;
	}
}

void stopServers(void)
{
	Server *server = Server::first;

	while(server)
	{
		server->Stop();
		server = server->next;
	}
}

PortManager::PortManager(Trunk *trk)
{
	trunk = trk;
	trunk->manager = this;
}

TGI::TGI()
{
	next = first;
	first = this;
}

TGI *getInterp(char *cmd)
{
	TGI *tgi = TGI::first;
	char *ext;
	char buffer[512];
	strcpy(buffer, cmd);
	cmd = strtok_r(buffer, " \t\n", &ext);
	ext = strrchr(cmd, '.');
	
	while(tgi)
	{
		if(tgi->getExtension(ext))
			return tgi;
		tgi = tgi->next;
	}
	return NULL;
}

void getInterp(char *cmd, char **args)
{
	TGI *tgi = TGI::first;

	while(tgi)
	{
		tgi->Script(cmd, args);
		tgi = tgi->next;
	}
}

Module::Module()
{
	modNext = modFirst;
	modFirst = this;
}

void Module::addSession(void)
{
	sesNext = sesFirst;
	sesFirst = this;
}

void Module::addNetwork(void)
{
	netNext = netFirst;
	netFirst = this;
}

void Module::addRequest(void)
{
	reqNext = reqFirst;
	reqFirst = this;
}

void Module::addCommand(void)
{
	cmdNext = cmdFirst;
	cmdFirst = this;
}

void Module::addPrompts(void)
{
	urlNext = urlFirst;
	urlFirst = this;
}

void detachModules(Trunk *trunk)
{
	Module *mod = Module::sesFirst;

	while(mod)
	{
		mod->Detach(trunk);
		mod = mod->sesNext;
	}
}

void attachModules(Trunk *trunk)
{
	Module *mod = Module::sesFirst;

	while(mod)
	{
		mod->Attach(trunk);
		mod = mod->sesNext;
	}
}

Module *getModule(modtype_t mtype, const char *name)
{
	Module *mod = Module::modFirst;

	while(mod)
	{
		if(mod->getType() == mtype || mtype == MODULE_ANY)
		{
			if(!name)
				return mod;

			if(!stricmp(name, mod->getName()))
				return mod;
		}
		mod = mod->modNext;
	}
	return NULL;
}

Debug *debug = NULL;
Driver *driver = NULL;
Module *asr = NULL;
Module *tts = NULL;
