// Copyright (C) 2000-2001 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "server.h"

bool Trunk::scrAssign(void)
{
	const char *var = getKeyword("var");
	const char *value = getKeyword("value");
	const char *size = getKeyword("size");

	if(!value)
		value = "";

	if(!size)
		setSymbol(var, getSymbolSize());
	else
		setSymbol(var, atoi(size));
	setSymbol(var, value);
	Advance();
	return true;
}

bool Trunk::scrDummy(void)
{
	Error("not-supported");
	return true;
}

bool Trunk::scrCleardigits(void)
{
	dtmf.bin.data[0] = 0;
	digits = 0;
	Advance();
	return true;
}

bool Trunk::scrIdle(void)
{
	idle_timer = atoi(getValue("0"));
	Advance();
	return true;
}

bool Trunk::scrSchedule(void)
{
	char cmd[65];
	bool ok = false;

	strcpy(cmd, "schedule ");
	strcat(cmd, getValue(""));
	if(!fifo.Command(cmd))
	{
		Error("schedule-failed");
		return true;
	}
	Advance();
	return true;
}

bool Trunk::scrTrim(void)
{
        scriptsymbol_t *sym;
        int size = 0;
        int len, l2;
        char *cp, *ep;
        const char *value = getMember();
	int offset = 0;

	if(value)
		offset = atoi(value);

	if(!stricmp(value, "end"))
		offset = 255;
 
        sym = getVariable(getSymbolSize());
        if(!sym)
        {
                Error("symbol-not-found");
                return true;
        }

        if(sym->flags.readonly)
        {
                Error("symbol-readonly");
                return true;
        }
 
        sym->data[sym->flags.size] = 0;
        len = strlen(sym->data);

	if(offset < 0)
		offset = len + offset;
	if(offset < 0)
		offset = 0;
	if(offset > len)
		offset = -1;

        while(len < sym->flags.size && NULL != (value = getValue(NULL)))
        {
                l2 = atoi(value);

		if(offset < 0)
		{
			if(l2 > len)
				l2 = len;

			offset = len - l2;
		}
 
		cp = sym->data;
		ep = sym->data + offset;
		while(l2-- && *ep)
			*(cp++) = *(ep++);
		*cp = 0;
                len = strlen(sym->data);
        }
 
        sym->flags.initial = false;
        if(sym->flags.commit)
                Commit(sym);
 
        Advance();
        return true;
}

bool Trunk::scrChop(void)
{
        scriptsymbol_t *sym;
        int size = 0;
        int len, l2;
        char *cp, *ep;
        const char *value = getMember();
	int offset = 0;

	if(value)
		offset = atoi(value);

	if(!stricmp(value, "end"))
		offset = 255;
 
        sym = getVariable(getSymbolSize());
        if(!sym)
        {
                Error("symbol-not-found");
                return true;
        }

        if(sym->flags.readonly)
        {
                Error("symbol-readonly");
                return true;
        }
 
        sym->data[sym->flags.size] = 0;
        len = strlen(sym->data);

	if(offset < 0)
		offset = len + offset;
	if(offset < 0)
		offset = 0;
	if(offset > len)
		offset = -1;

        while(len < sym->flags.size && NULL != (value = getValue(NULL)))
        {
                l2 = atoi(value);

		if(offset < 0)
		{
			if(l2 > len)
				l2 = len;

			offset = len - l2;
		}
 
                cp = sym->data + offset;
                ep = sym->data + offset + l2;
		while(*ep)
			*(cp++) = *(ep++);
		*cp = 0;
                len = strlen(sym->data);
        }
 
        sym->flags.initial = false;
        if(sym->flags.commit)
                Commit(sym);
 
        Advance();
        return true;
}


bool Trunk::scrPrefix(void)
{
        scriptsymbol_t *sym;
        int size = 0;
        int len, l2;
        char *cp, *ep;
        const char *value = getMember();
	int max = 0;

	if(value)
		max = atoi(value);
 
        sym = getVariable(getSymbolSize());
        if(!sym)
        {
                Error("symbol-not-found");
                return true;
        }

        if(sym->flags.readonly)
        {
                Error("symbol-readonly");
                return true;
        }
 
        sym->data[sym->flags.size] = 0;
        len = strlen(sym->data);

	if(!max)
		max = sym->flags.size;

        while(len < sym->flags.size && NULL != (value = getValue(NULL)))
        {
                l2 = strlen(value);
                if(l2 + len > max)
                        continue;
 
                if(!strnicmp(sym->data, value, l2))
                                continue;
 
                cp = sym->data + len;
                ep = cp + l2;
                while(cp > sym->data)
                        *(--ep) = *(--cp);
 
                memcpy(sym->data, value, l2);
                sym->data[len + l2] = 0;
                break;
        }
 
        sym->flags.initial = false;
        if(sym->flags.commit)
                Commit(sym);
 
        Advance();
        return true;
}

bool Trunk::scrInsert(void)
{
        scriptsymbol_t *sym;
        int size = 0;
        int len, l2;
        char *cp, *ep;
        const char *value = getMember();
	int offset = 0;

	if(value)
		offset = atoi(value);
 
        sym = getVariable(getSymbolSize());
        if(!sym)
        {
                Error("symbol-not-found");
                return true;
        }

        if(sym->flags.readonly)
        {
                Error("symbol-readonly");
                return true;
        }
 
        sym->data[sym->flags.size] = 0;
        len = strlen(sym->data);

	if(offset < 0)
		offset = len + offset;
	if(offset < 0)
		offset = 0;
	if(offset > len)
		offset = len;

        while(len < sym->flags.size && NULL != (value = getValue(NULL)))
        {
                l2 = strlen(value);
		if(len + l2 > sym->flags.size)
			l2 = sym->flags.size - len;
 
                cp = sym->data + len;
                ep = cp + l2;
                while(cp > sym->data + offset)
                        *(--ep) = *(--cp);
 
                memcpy(sym->data + offset, value, l2);
                sym->data[len + l2] = 0;
                offset += l2;
		len += l2;
        }
 
        sym->flags.initial = false;
        if(sym->flags.commit)
                Commit(sym);
 
        Advance();
        return true;
}

bool Trunk::scrDelete(void)
{
        scriptsymbol_t *sym;
        int size = 0;
        int len, l2;
        char *cp, *ep;
        const char *value = getMember();
	int offset = 0;

	if(value)
		offset = atoi(value);

	if(!stricmp(value, "end"))
		offset = 255;
 
        sym = getVariable(getSymbolSize());
        if(!sym)
        {
                Error("symbol-not-found");
                return true;
        }

        if(sym->flags.readonly)
        {
                Error("symbol-readonly");
                return true;
        }
 
        sym->data[sym->flags.size] = 0;
        len = strlen(sym->data);

	if(offset < 0)
		offset = len + offset;
	if(offset < 0)
		offset = 0;
	if(offset > len)
		offset = -1;

        while(len < sym->flags.size && NULL != (value = getValue(NULL)))
        {
                l2 = strlen(value);

		if(offset < 0)
		{
			if(l2 > len)
				continue;

			if(strnicmp(sym->data + len - l2, value, l2))
				continue;
			offset = len - l2;
		}
		else
		{
			if(strnicmp(sym->data + offset, value, l2))
				continue;
		} 		
 
                cp = sym->data + offset;
                ep = sym->data + offset + l2;
		while(*ep)
			*(cp++) = *(ep++);
		*cp = 0;
                break;
        }
 
        sym->flags.initial = false;
        if(sym->flags.commit)
                Commit(sym);
 
        Advance();
        return true;
}

bool Trunk::scrReplace(void)
{
        scriptsymbol_t *sym;
        int size = 0;
        int len, l2, l3;
        char *cp, *ep;
	char *rep;
        const char *value = getMember();
	int offset = 0;

	if(value)
		offset = atoi(value);

	if(!stricmp(value, "end"))
		offset = 255;
 
        sym = getVariable(getSymbolSize());
        if(!sym)
        {
                Error("symbol-not-found");
                return true;
        }

        if(sym->flags.readonly)
        {
                Error("symbol-readonly");
                return true;
        }
 
        sym->data[sym->flags.size] = 0;
        len = strlen(sym->data);

	if(offset < 0)
		offset = len + offset;
	if(offset < 0)
		offset = 0;
	if(offset > len)
		offset = -1;

        while(len < sym->flags.size && NULL != (value = getValue(NULL)))
        {
                l2 = strlen(value);
		rep = getValue(NULL);
		if(rep)
			l3 = strlen(rep);
		else
			l3 = 0;

		if(offset < 0)
		{
			if(l2 > len)
				continue;

			if(strnicmp(sym->data + len - l2, value, l2))
				continue;
			offset = len - l2;
		}
		else
		{
			if(strnicmp(sym->data + offset, value, l2))
				continue;
		} 		
 
		if(l3 > l2)
		{
			cp = sym->data + len;
			while((cp--) >= sym->data + offset)
			{
				cp[l3 - l2] = *cp;
			}
			sym->data[len + l3 - l2] = 0;
		}
		else if(l3 < l2)
		{
			cp = sym->data + offset + l2 - l3;
			ep = sym->data + offset;
			while(*cp)
				*(ep++) = *(cp++);
			*ep = 0;
		}
		if(offset + l3 >= len)
			strcpy(sym->data + offset, rep);
		else 	
			memcpy(sym->data + offset, rep, l3);
                break;
        }
 
        sym->flags.initial = false;
        if(sym->flags.commit)
                Commit(sym);
 
        Advance();
        return true;
}

bool Trunk::scrSignal(void)
{
	Trunk *trunk;
	TrunkEvent event;

	trunk = driver->getTrunkPort(atoi(getValue("-1")));
	if(!trunk)
	{
		Error("signal-no-such-trunk");
		return true;
	}

	event.id = TRUNK_SIGNAL_NOTIFY;
	event.parm.error = getValue(NULL);
	if(!trunk->postEvent(&event))
	{
		Error("signal-not-waiting");
		return true;
	}
	Advance();
	return true;
}

bool Trunk::scrModule(void)
{
	scriptline_t *line = getScript();
	char *cmd = line->cmd;
	char keybuf[33];
	int len = 0;
	char *kw = keybuf;

	while(len++ < 32 && *cmd && *cmd != '.')
		*(kw++) = *(cmd++);
	*kw = 0;

	Module *module = getModule(MODULE_ANY, keybuf);
	char *err;
	unsigned delay;

	if(!module)
	{
		Error("module-not-found");
		return true;
	}

	err = module->Dispatch(this);
	if(err)
	{
		Error(err);
		return true;
	}

	delay = module->Sleep(this);
	if(!delay)
	{
		Advance();
		return true;
	}
	data.sleep.wakeup = delay * 1000;
	data.sleep.rings = 0;
	data.sleep.loops = 1;
	TrunkStep(TRUNK_STEP_SLEEP);
	return false;
}

bool Trunk::scrRecord(void)
{
	const char *member = getMember();
	const char *cp;

	if(!member)
		member="all";

	data.record.name = getValue("");	
	if(!data.record.name)
	{
		Error("record-no-file");
		return true;
	}

	data.record.timeout = getTimeout();
	data.record.term = getDigitMask();
	data.record.offset = 0;
	data.record.volume = atoi(getSymbol(SYM_VOLUME));
	data.record.trim = atoi(getSymbol(SYM_TRIM));
	
	if(!stricmp(member, "append"))
		data.record.append = true;
	else
		data.record.append = false;
	if(NULL != (cp = getKeyword("offset")))
		data.record.offset = atoi(cp);
	if(NULL != (cp = getKeyword("volume")))
		data.record.volume = atoi(cp);

	TrunkStep(TRUNK_STEP_RECORD);
	return false;
}

bool Trunk::scrMap(void)
{
	char *map = getValue(NULL);
	char *key = getValue(NULL);
	const char *prefix = getMember();
	char **list, **tok;
	unsigned len, lkey;
	mapmode_t mapmode = MAP_PREFIX;
	
	if(prefix)
	{
		if(!strnicmp(prefix, "pre", 3))
			mapmode = MAP_PREFIX;
		else if(!strnicmp(prefix, "suf", 3))
			mapmode = MAP_SUFFIX;
		else if(!strnicmp(prefix, "abs", 3))
			mapmode = MAP_ABSOLUTE;
	}

	if(!key)
	{
		Error("no-key-value");
		return true;
	}

	lkey = strlen(key);

	while(NULL != (prefix = getValue(NULL)))
	{
		len = strlen(prefix);
		if(!strnicmp(prefix, key, len))
		{
			key += len;
			break;
		}
		break;
	}
	
	initKeywords(0);
	list = getMap(map, key, mapmode);
	if(!list)
	{
		Advance();
		return true;
	}

	if(!Redirect(*list))
	{
		Error("map-script-missing");
		return true;
	}
	setList(++list);
	return true;
}

bool Trunk::scrTransfer(void)
{
	strstream str(data.dialxfer.digits, sizeof(data.dialxfer.digits) - 1); 
	char *cp = (char *)group->getLast("transfer");
	if(cp)
		str << cp;

	while(NULL != (cp = getValue(NULL)))
		str << cp;

	str << ends;
	data.dialxfer.digits[sizeof(data.dialxfer.digits) - 1] = 0;
	data.dialxfer.interdigit = group->getDialspeed();
	data.dialxfer.digit = data.dialxfer.digits;
	data.dialxfer.exit = true;
	data.dialxfer.timeout = 0;
	data.dialxfer.onhook = atol(getValue(group->getLast("flash")));
        data.dialxfer.offhook = atol(getValue(group->getLast("dialtone")));      	
	TrunkStep(TRUNK_STEP_FLASH);
	return false;
}

bool Trunk::scrHold(void)
{
	const char *cp = group->getLast("hold");
	if(!cp)
		cp = "";

	strcpy(data.dialxfer.digits, cp);
	data.dialxfer.interdigit = group->getDialspeed();
        data.dialxfer.digit = data.dialxfer.digits;
        data.dialxfer.exit = true;
        data.dialxfer.timeout = 0;
        data.dialxfer.onhook = atol(getValue(group->getLast("flash")));
        data.dialxfer.offhook =atol(getValue(group->getLast("dialtone")));
	TrunkStep(TRUNK_STEP_FLASH);
	return false;
}

bool Trunk::scrSpeak(void)
{
	Translator *tts;
	char *err;
	const char *member = getMember();

	if(!member)
		member="all";

	if(!stricmp(member, "any"))
		data.play.mode = PLAY_MODE_ANY;
	else
		data.play.mode = PLAY_MODE_NORMAL;

	data.play.term = 0;

	tts = getTranslator(getSymbol(SYM_LANGUAGE));

	if(!tts)
	{
		Error("language-unsupported");
		return true;
	}
	err = tts->Speak(this);
	if(err)
	{
		Error(err);
		return true;
	}
	while(*data.play.name == ',')
		++data.play.name;

	data.play.timeout = 0;
	data.play.repeat = 0;
	data.play.volume = atoi(getSymbol(SYM_VOLUME));
	TrunkStep(TRUNK_STEP_PLAY);
	return false;
}
	
bool Trunk::scrDial(void)
{
	strstream str(data.dialxfer.digits, sizeof(data.dialxfer.digits) - 1); 
	char *cp;

	while(NULL != (cp = getValue(NULL)))
		str << cp;

	str << ends;
	data.dialxfer.digits[sizeof(data.dialxfer.digits) - 1] = 0;
	data.dialxfer.interdigit = group->getDialspeed();
	data.dialxfer.digit = data.dialxfer.digits;
	data.dialxfer.exit = false;
	data.dialxfer.timeout = group->getAnalysis() * 1000;
	TrunkStep(TRUNK_STEP_DIALXFER);
	return false;
}

bool Trunk::scrDTMF(void)
{
	strstream str(data.dialxfer.digits, sizeof(data.dialxfer.digits) - 1); 
	char *cp;

	while(NULL != (cp = getValue(NULL)))
		str << cp;

	str << ends;
	data.dialxfer.digits[sizeof(data.dialxfer.digits) - 1] = 0;
	data.dialxfer.interdigit = group->getDialspeed();
	data.dialxfer.digit = data.dialxfer.digits;
	data.dialxfer.exit = false;
	data.dialxfer.timeout = 0;
	TrunkStep(TRUNK_STEP_DIALXFER);
	return false;
}

bool Trunk::scrSay(void)
{
	strstream str(data.play.list, sizeof(data.play.list) - 1);
	char *cp;

	while(NULL != (cp = getValue(NULL)))
		str << "+" << cp;

	str << ends;
	if(!data.play.list[0])
	{
		Error("tts-no-output");
		return true;
	}
	data.play.list[sizeof(data.play.list) - 1] = 0;
	libtts(data.play.list + 1, TTS_GATEWAY_TEXT);
	sprintf(data.play.list, "temp/.tts.%d.ul", id);
	data.play.name = data.play.list;
	data.play.repeat = 0;
	data.play.timeout = ::getTimeout(getSymbol(SYM_PLAYWAIT));
	data.play.volume = atoi(getSymbol(SYM_VOLUME));
	data.play.term = 0;
	data.play.offset = 0;
	data.play.mode = PLAY_MODE_TEMP;
	data.play.limit = data.play.offset = 0;

	if(NULL != (cp = getKeyword("offset")))
		data.play.offset = atol(cp);
	if(NULL != (cp = getKeyword("volume")))
		data.play.volume = atoi(cp);
	if(NULL != (cp = getKeyword("timeout")))
		data.play.timeout = ::getTimeout(cp);
	TrunkStep(TRUNK_STEP_PLAYWAIT);
	return false;
}

bool Trunk::scrLoad(void)
{
	Module *mod;
	const char *kw = getMember();
	scriptline_t *line = getScript();
	int count;
	char *sym;
	char *value;
	char *str;
	char *var;
	int len;

	if(!kw)
		kw = "put";

	if(!stricmp(kw, "xml"))
	{
		var = getValue("");
		if(altimage)
		{
			altimage->Purge();
			delete altimage;
			altimage = NULL;
		}
		mod = getModule(MODULE_XML, var);
		if(mod)
			altimage = mod->getXML();
	}
 
	if(!altimage)
	{
		Error("no-xml-parser");
		return true;
	}

	if(!stricmp(kw, "xml"))
	{
		Advance();
		return true;
	}

	altimage->Purge();
	if(!stricmp(kw, "post"))
		data.load.post = true;
	else
		data.load.post = false;
	kw = getKeyword("section");
	if(!kw)
		kw = "#";
	if(*kw != '#')
	{
		var = (char *)altimage->alloc(strlen(kw + 1));
		var[0] = '#';
		strcpy(var + 1, ++kw);
		kw = var;
	}
	data.load.section = kw;
	kw = getKeyword("maxTime");	
	if(!kw)
		kw = "60";
	data.load.image = altimage;
	data.load.timeout = ::getTimeout(kw);
	count = line->argc + 1;
	data.load.vars = (char **)altimage->alloc(count * sizeof(char *));
	data.load.url = getValue(NULL);
	if(!data.load.url)
	{
		Error("no-xml-url");
		return true;
	}

	if(!strnicmp(data.load.url, "http:", 5))
		altimage->setProxy(keyproxy.getHTTPServer(), keyproxy.getHTTPPort());
	else
		altimage->setProxy(NULL, 0);
		
	count = 0;
	while(NULL != (sym = getOption(NULL)))
	{
		value = getContent(sym);
		if(*sym == '%' || *sym == '@')
			++sym;
		len = (strlen(sym) + strlen(value) * 2);
		var = (char *)altimage->alloc(len);
		data.load.vars[count++] = var;
		urlencode(sym, var, len);
		strcat(var, "=");
		len -= strlen(var);
		var += strlen(var);
		urlencode(sym, value, len);
	}
	data.load.vars[count] = NULL;
	if(NULL != (kw = getKeyword("timeout")))
		data.load.timeout = atoi(kw) * 1000;		
	TrunkStep(TRUNK_STEP_LOADER);
	return false;
}

bool Trunk::scrSend(void)
{
	int port = atoi(getValue(getSymbol("id")));
	char *sym, *value;
	scriptsymbol_t *s;

	Trunk *trunk = driver->getTrunkPort(port);

	if(!trunk)
	{
		Error("invalid-trunk");
		return true;
	}

	while(NULL != (sym = getOption(NULL)))
	{
		trunk->EnterMutex();
		if(*sym != '%')
		{
			trunk->LeaveMutex();
			Error("symbol-invalid");
			return true;
		}
		s = trunk->getEntry(++sym, 0);
		if(!s)
		{
			trunk->LeaveMutex();
			Error("symbol-missing");
			return true;
		} 
		value = getValue("");
		if(!trunk->postSymbol(s, value))
		{
			trunk->LeaveMutex();
			Error("invalid-symbol");
			return true;
		}
		trunk->LeaveMutex();
	}
	Advance();
	return true;
}

bool Trunk::scrPlay(void)
{
	scriptname_t *scr;
	strstream str(data.play.list, sizeof(data.play.list) - 1);
	char *cp;
	unsigned long mask = 0;

	const char *member = getMember();
	if(!member)
		member = "all";

	if(!stricmp(member, "any"))
		data.play.mode = PLAY_MODE_ANY;
	else if(!stricmp(member, "one"))
		data.play.mode = PLAY_MODE_ONE;
	else if(!stricmp(member, "temp") || !stricmp(member, "tmp"))
		data.play.mode = PLAY_MODE_TEMP;
	/* else if(0 != (mask = getScriptMask(member)))
		data.play.mode = PLAY_MODE_ANY; */
	else
		data.play.mode = PLAY_MODE_NORMAL;

	if(mask)
	{
		scr = getObject();
		if((mask & scr->mask) != mask)
		{
			Advance();
			return true;
		}
	}	

	while(NULL != (cp = getValue(NULL)))
		str << "," << cp;

	str << ends;
	data.play.list[sizeof(data.play.list) - 1] = 0;
	data.play.name = data.play.list;
	data.play.volume = atoi(getSymbol(SYM_VOLUME));
	data.play.timeout = 0;
	data.play.repeat = 0;
	data.play.term = 0;
	while(*data.play.name == ',')
		++data.play.name;

	if(!*data.play.name)
	{
		Error("play-no-files");
		return true;
	}

	data.play.limit = data.play.offset = 0;

	if(NULL != (cp = getKeyword("offset")))
		data.play.offset = atol(cp);

	if(NULL != (cp = getKeyword("volume")))
		data.play.volume = atoi(cp);

	TrunkStep(TRUNK_STEP_PLAY);
	return false;
}	

bool Trunk::scrFlash(void)
{
	data.dialxfer.onhook = atol(getValue(group->getLast("flash")));
	data.dialxfer.offhook = atol(getValue(group->getLast("dialtone")));
	data.dialxfer.digit = NULL;
	TrunkStep(TRUNK_STEP_FLASH);
	return false;
}

bool Trunk::scrCollect(void)
{
	data.collect.count = atoi(getValue("0"));
	if(data.collect.count > 32)
		data.collect.count = 32;

	data.collect.timeout = getInterdigit();
	data.collect.term = getDigitMask();
	data.collect.ignore = getDigitMask();
	TrunkStep(TRUNK_STEP_COLLECT);
	return false;
}

bool Trunk::scrAccept(void)
{
	if (group->getAccept())
	{
		Advance();
		return true;
	}

	TrunkStep(TRUNK_STEP_ACCEPT);
	return false;
}
	
bool Trunk::scrAnswer(void)
{
	const char *kw;

	if(NULL != (kw = getKeyword("maxRing")))
		data.answer.rings = atoi(kw);
	else 	
		data.answer.rings = atoi(getValue("0"));
	if(NULL != (kw = getKeyword("maxTime")))
		data.answer.timeout = ::getTimeout(kw);
	else
		data.answer.timeout = ::getTimeout(getValue(group->getLast("ringtime")));
	TrunkStep(TRUNK_STEP_ANSWER);
	return false;
}	

bool Trunk::scrHangup(void)
{
	if(!Signal((unsigned int)0))
		scrExit();

	return true;
}

bool Trunk::scrAudit(void)
{
	cdrc = 0;
	char *val;

	while(cdrc < 32)
	{
		val = getOption(NULL);
		if(!val)
			break;

		cdrv[cdrc++] = val;
	}
	Advance();
	return true;
}

bool Trunk::scrAlog(void)
{
	char buffer[256];
	strstream str(buffer, 256);
	char *tag, *val;

	buffer[0] = 0;
	while(NULL != (tag = getOption(NULL)))
	{
		val = getContent(tag);
		if(!val)
			continue;

		if(!*val)
			continue;

		if(!strnicmp(tag, "%cdr::", 6))
			tag += 6;
		else if(!strnicmp(tag, "%alog::", 7))
			tag += 7;
		else if(!strnicmp(tag, "%cdr_", 5))
			tag += 5;
		else
			++tag;

		str << " " << tag << "=" << val;
	}
	str << ends;
	alog(this, buffer + 1);
	Advance();
	return true;
}

bool Trunk::scrImport(void)
{
	scriptsymbol_t *sym;
	char *name;
	char *value;

	while(NULL != (name = getOption(NULL)))
	{
		if(*name++ != '%')
			continue;

		value = globals.getSymbol(name);
		if(!value)
			continue;

		setSymbol(name, getSymbolSize());
		setSymbol(name, value);
	}
	Advance();
	return true;
}

bool Trunk::scrDebug(void)
{
	char buf[256];
	char *value;

	buf[0] = 0;
	while(NULL != (value = getValue(NULL)))
		strcat(buf, value);
	debug->DebugScript(this, buf);
	Advance();
	return true;
}

bool Trunk::scrExport(void)
{
	scriptsymbol_t *sym;
	char *name;

	while(NULL != (name = getOption(NULL)))
	{
		if(*name != '%')
			continue;

		sym = getEntry(++name, 0);
		if(!sym)
			continue;

		globals.setSymbol(name, sym->flags.size);
		globals.setSymbol(name, sym->data);
	}
	Advance();
	return true;
}

bool Trunk::scrSync(void)
{
	timeout_t timer = getTimeout();
	time_t now;

	time(&now);
	timer = timer - (now - start);
	if(timer < 1)
	{
		Advance();
		return true;
	}
	data.sleep.wakeup = timer;
	data.sleep.rings = atoi(getValue("0"));
	data.sleep.loops = 1;
	TrunkStep(TRUNK_STEP_SLEEP);
	return false;
}

bool Trunk::scrPause(void)
{
	data.sleep.wakeup = atol(getValue("0"));
	data.sleep.loops = 1;
	if(data.sleep.wakeup < 20)
		data.sleep.wakeup = 20;

	data.sleep.rings = 0;
	TrunkStep(TRUNK_STEP_SLEEP);
	return false;
}

bool Trunk::scrStart(void)
{
	char buffer[512];
	strstream str(buffer, sizeof(buffer));
	char *arg = getValue("0");
	unsigned port = atoi(arg);
	TrunkGroup *grp = getGroup(arg);

	if(!port && !grp)
	{
		Error("start-no-channel-or-group");
		return true;
	}
	arg = getValue(NULL);
	if(!arg)
	{
		Error("start-no-script");
		return true;
	}
	port += atoi(getSymbol(SYM_ID));
	buffer[0] = 0;
	if(grp)
		str << "start " << grp->getName() << " " << arg;
	else
		str << "start " << port << " " << arg;

	while(NULL != (arg = getOption(NULL)))
	{
		if(*arg != '%')
			continue;	

		str << " " << (arg + 1)	<< "=" << getContent(arg);
	}
	str << ends;

	if(!fifo.Command(buffer))
	{
		Error("start-failed");
		return true;
	}
	Advance();
	return true;
}

bool Trunk::scrRequest(void)
{
	TrunkGroup *grp;
	int argc = 0;
	char *argv[32];
	char *arg;
	unsigned expires = atoi(getValue("0"));
	
	arg = getValue(NULL);
	if(!arg)
	{
		Error("request-no-group");
		return true;
	}
	grp = getGroup(arg);
	if(!grp)
	{
		Error("request-unknown-group");
		return true;
	}
	arg = getValue(NULL);
	if(!arg)
	{
		Error("request-no-script");
		return true;
	}
	argv[argc++] = arg;
	while(NULL != (argv[argc] = getOption(NULL)))
	{
		if(argc > 31)
			break;
		++argc;
	}
	argv[argc] = NULL;

	request(grp, argv, expires);
	Advance();
	return true;
}	

bool Trunk::scrLibexec(void)
{
	tgicmd_t cmd;
	char query[sizeof(cmd.cmd) / 2];
	strstream str(cmd.cmd, sizeof(cmd.cmd));
	strstream q(query, sizeof(query));
	char *qc, *tag;
	const char *member = getMember();
	if(!member)
		member="tgi";

	if(!strnicmp(member, "one", 3))
	{
		if(!getOnce())
		{
			Advance();
			return true;
		}
	}

	int cnt = 0;
	int args = 0;

	if(!stricmp(member, "play"))
		data.sleep.wakeup = ::getTimeout(getValue(getSymbol(SYM_PLAYWAIT)));
	else
		data.sleep.wakeup = getTimeout();
	data.sleep.rings = 0;
	data.sleep.loops = 1;

	cmd.port = id;
	cmd.mode = TGI_EXEC_NORMAL;
	cmd.cmd[0] = 0;
	query[0] = 0;
	
	str << getValue("--");
	while(NULL != (qc = getOption(NULL)))
	{
		if(*qc != '%')
			continue;

		if(!strnicmp(qc, "%lib::", 6))
			tag = qc + 6;
		else if(!strnicmp(qc, "%tgi::", 6))
			tag = qc + 6;
		else if(!strnicmp(qc, "%lib_", 5))
			tag = qc + 5;
		else if(!strnicmp(qc, "%lib.", 5))
			tag = qc + 5;
		else
			tag = qc + 1;

		if(!query[0])
			q << tag << "=" << getContent(qc);
		else
			q << keyserver.getToken() << tag << "=" << getContent(qc);
	}
	q << ends;
	str << " query=" << query;
	
	str << " digits=" << dtmf.bin.data;		

	qc = getSymbol(SYM_CALLER);
	if(qc)
		str << " clid=" << qc;

	qc = getSymbol(SYM_DIALED);
	if(qc)
		str << " dnid=" << qc;

	if(!data.sleep.wakeup)
		cmd.mode = TGI_EXEC_DETACH;

	if(!stricmp(member, "play"))
	{
		data.play.timeout = data.sleep.wakeup;
		data.play.repeat = 0;
		data.play.name = data.play.list;
		data.play.term = 0;
		data.play.mode = PLAY_MODE_TEMP;
		data.play.limit = data.play.offset = 0;
		data.play.volume = atoi(getSymbol(SYM_VOLUME));
		sprintf(data.play.list, "temp/.tmp.%d.%s",
			id, getSymbol(SYM_EXTENSION));
		str << " audio=" << data.play.name;
	}

	str << ends;
	::write(tgipipe[1], &cmd, sizeof(cmd));

	if(!stricmp(member, "play"))
	{
		TrunkStep(TRUNK_STEP_PLAYWAIT);
		return false;
	}

	if(!data.sleep.wakeup)
	{
		Advance();
		return true;
	}
		
	TrunkStep(TRUNK_STEP_SLEEP);
	return false;
}

bool Trunk::scrTone(void)
{
	phTone *tone = getphTone(getValue(""));
	if(!tone)
	{
		Error("no-tone");
		return true;
	}
	data.tone.tone = tone;
	data.tone.wakeup = atol(getValue("0"));
	data.tone.loops = atoi(getValue("1"));
	TrunkStep(TRUNK_STEP_TONE);
	return false;
}


bool Trunk::scrSleep(void)
{
	timeout_t timer = getTimeout();

	if(!timer)
	{
		Advance();
		return true;
	}

	data.sleep.wakeup = timer;
	data.sleep.loops = 1;
	data.sleep.rings = 0;
	TrunkStep(TRUNK_STEP_SLEEP);
	return false;
}

bool Trunk::scrFunction(void)
{
	char *name = getOption(NULL);
	scriptsymbol_t *sym = getEntry(name, getSymbolSize());
	functioncall_t fun = driver->getFunction(name);
	char *argv[32];
	int count = 0;

	if(!fun)
	{
		Error("function-not-found");
		return true;
	}

	if(!sym)
	{
		Error("symbol-not-found");
		return true;
	}

	if(sym->flags.readonly)
	{
		Error("symbol-readonly");
		return true;
	}

	sym->data[sym->flags.size] = 0;
	while(count < 31)
	{
		argv[count] = getValue();
		if(!argv[count])
			break;
		++count;
	}
	argv[count] = NULL;
	name = (*fun)(sym, argv);
	if(name)
		Error(name);
	else
	{
		sym->flags.initial = false;
		if(sym->flags.commit)
			Commit(sym);
		Advance();
	}
	return true;
}
