// Copyright (C) 2000-2001 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <cc++/config.h>
#include <getopt.h>
#include <sys/wait.h>
#include <sys/utsname.h>
#include <cerrno>
#include "server.h"

#ifdef	_POSIX_MEMLOCK
#include <sys/mman.h>
#endif

bool aliases = false;
bool running = false;
int mainpid;
int tgipipe[2];
bool restart_server = false;
static int *tgipids = NULL;
static Resolver *resolver = NULL;
static char **restart_argv;
static char **cmds;

#ifndef	HAVE_SETENV
static void setenv(char *name, const char *value, int overwrite)
{
	char strbuf[256];

	snprintf(strbuf, sizeof(strbuf), "%s=%s", name, value);
	putenv(strbuf);
}
#endif

static void purge(char *path)
{
	char *ex[4];
	int pid;

	pid = fork();
	if(pid < 0)
		return;

	if(pid)
	{
		waitpid(pid, NULL, 0);
		return;
	}

	ex[0] = "rm";
	ex[1] = "-rf";
	ex[2] = path;
	ex[3] = NULL;

	exit(execvp("rm", ex));
}
		
static void rt(void)
{
	int min, max;
	int pages = keythreads.getPages();
	int pri = keythreads.getPriority();
	int policy = keythreads.getPolicy();
	char memory[1024 * pages];

#ifdef	_POSIX_PRIORITY_SCHEDULING
	struct sched_param p;
	
	sched_getparam(0, &p);
	pri += p.sched_priority;
	if(!policy)
		policy = sched_getscheduler(0);

	min = sched_get_priority_min(policy);
	max = sched_get_priority_max(policy);
	if(pri < min)
		pri = min;
	if(pri > max)
		pri = max;
	p.sched_priority = pri;
	sched_setscheduler(getpid(), policy, &p); 
#else
	nice(-pri);
#endif

#ifdef	MCI_CURRENT
	if(pages)
		mlockall(MCI_CURRENT | MCI_FUTURE);
#endif
}

static RETSIGTYPE tgiusr1(int signo)
{
}

static void tgi(void)
{
	int argc;
	char *argv[65];
	char buffer[512];
	int cfd, vpid, pool, stat;
	int count = keythreads.getGateways();
	tgipids = new int[count];
	sigset_t sigs;
	struct sigaction act;
	tgicmd_t cmd;
	TGI *tgi;
	int status;
	char *cp;
	int len;
	char **arg;

	if(canModify(keypaths.getRunfiles()))
	{
		strcpy(buffer, keypaths.getRunfiles());
		strcat(buffer, "/bayonne");
		setenv("SERVER_CONTROL", buffer, 1);
		strcpy(buffer, keypaths.getRunfiles());
		strcat(buffer, "/bayonne.ctrl");
	}
	else
	{
		sprintf(buffer, "%s/.bayonne", getenv("HOME"));
		setenv("SERVER_CONTROL", buffer, 1);
		sprintf(buffer, "%s/.bayonne.ctrl", getenv("HOME"));
	}

	pipe(tgipipe);

	for(pool = 0; pool < count; ++pool)
	{
#ifdef	__FreeBSD__
		tgipids[pool] = vfork();
#else
		tgipids[pool] = fork();
#endif
		if(!tgipids[pool])
			break;
	}
	
	if(pool == count)
	{
		close(tgipipe[0]);
		return;
	}

	close(tgipipe[1]);

	arg = cmds;
	while(*arg)
	{	
		len = strlen(*arg);
		memset(*arg, 0, len);
		++arg;
	}
	strcpy(cmds[0], "[tgi]");
	keyserver.setGid();
	keyserver.setUid();
	sigemptyset(&sigs);
	sigaddset(&sigs, SIGINT);
	signal(SIGINT, SIG_DFL);
	sigprocmask(SIG_UNBLOCK, &sigs, NULL);

	act.sa_handler = &tgiusr1;
	act.sa_flags = SA_RESTART;
	sigemptyset(&act.sa_mask);
	sigaction(SIGUSR1, &act, NULL);	

	slog(SLOG_NOTICE) << "tgi: initialized; uid=" << getuid() << " pid=" << getpid() << endl;
	nice(-keythreads.priGateway());

	count = 6;
	while(count--)
	{
		sleep(10);
		cfd = open(buffer, O_RDWR);
		if(cfd > -1)
			break;
	}
	while(cfd > -1 && cfd < 3)
		cfd = dup(cfd);
	if(cfd < 0)
		slog(SLOG_ERROR) << "tgi: fifo failed; buffer=" << buffer << "; failure=" << errno << endl;
	else
		slog(SLOG_DEBUG) << "tgi: buffer=" << buffer << "; cfd=" << cfd << "; time=" << count * 10 << endl;

	while(read(tgipipe[0], &cmd, sizeof(cmd)) == sizeof(cmd))
	{
		argc = 0;
		slog(SLOG_DEBUG) << "tgi: cmd=" << cmd.cmd << endl;
		tgi = getInterp(cmd.cmd);
		if(tgi)
		{
			if(cmd.mode != TGI_EXEC_DETACH)
				sprintf(buffer, "wait %d %d\n", cmd.port, getpid());
			status = tgi->Parse(cfd, cmd.port, cmd.cmd);
			if(cmd.mode != TGI_EXEC_DETACH)
				sprintf(buffer, "exit %d %d\n", cmd.port, status);
			continue;
		}
		vpid = vfork();
		if(vpid)
		{
			if(cmd.mode != TGI_EXEC_DETACH)
			{
				sprintf(buffer, "wait %d %d\n", cmd.port, vpid);
				write(cfd, buffer, strlen(buffer));
				if(keythreads.getAudit())
					slog(SLOG_DEBUG) << "thread: gateway waiting on pid " << vpid << endl;
			}
			else
			{
				if(keythreads.getAudit())
					slog(SLOG_DEBUG) << "thread: gateway detached pid " << vpid << endl;
			}

			if(cmd.mode != TGI_EXEC_DETACH)
			{
#ifdef	__FreeBSD__
				wait4(vpid, &stat, 0, NULL);
#else
				waitpid(vpid, &stat, 0);
#endif
				if(keythreads.getAudit())
					slog(SLOG_DEBUG) << "thread: gateway exiting pid " << vpid << endl;
				sprintf(buffer, "exit %d %d\n", cmd.port, WEXITSTATUS(stat));
				write(cfd, buffer, strlen(buffer));
			}
			else
#ifdef	__FreeBSD__
				wait4(vpid, &stat, 0, NULL);
#else
				waitpid(vpid, &stat, 0);
#endif
			continue;
		}

		if(cmd.mode == TGI_EXEC_DETACH)
		{
			close(0);
                	close(1);
               		close(2);
                	pdetach();
                	open("/dev/null", O_RDWR);
                	open("/dev/null", O_RDWR);
                	open("/dev/null", O_RDWR);
			strcpy(cmds[0], "-detach");
		}
		else
			sprintf(cmds[0], "tgi/%03d", cmd.port);

		sprintf(buffer, "%d", cmd.port);
		setenv("PORT_NUMBER", buffer, 1);
		argv[argc++] = strtok(cmd.cmd, " \t");
		while(NULL != (argv[argc] = strtok(NULL, " \t")))
		{
			/* Used for standard TGI execution */

			if(!strnicmp(argv[argc], "digits=", 7))
			{
				setenv("PORT_DIGITS", argv[argc] + 7, 1);
				continue;
			}

			if(!strnicmp(argv[argc], "dnid=", 5))
			{
				setenv("PORT_DNID", argv[argc] + 5, 1);
				continue;
			}

			if(!strnicmp(argv[argc], "clid=", 5))
			{
				setenv("PORT_CLID", argv[argc] + 5, 1);
				continue;
			}

			if(!strnicmp(argv[argc], "query=", 6))
			{
				setenv("PORT_QUERY", argv[argc] + 6, 1);
				continue;
			}

			// Added for TTS audio format selection

			if(!strnicmp(argv[argc], "format=", 6))
			{
				setenv("TTS_FORMAT", argv[argc] + 6, 1);
				continue;
			}

			// Added for TTS audio file type and ASR filename

			if(!strnicmp(argv[argc], "audio=", 6))
			{
				if(cmd.mode == TGI_EXEC_AUDIO)
				{
					remove(argv[argc] + 6);
					setenv("TTS_AUDIO", argv[argc] + 6, 1);
				}
				else
					setenv("ASR_AUDIO", argv[argc] + 6, 1);
				setenv("TEMP_AUDIO", argv[argc] + 6, 1);
				continue;
			}

			// Added for TTS/ASR language specification

			if(!strnicmp(argv[argc], "language=", 9))
			{
				setenv("TTS_LANGUAGE", argv[argc] + 9, 1); 
				setenv("ASR_LANGUAGE", argv[argc] + 9, 1);
				continue;
			}

			// Added for TTS/ASR voice domain selection

			if(!strnicmp(argv[argc], "voice=", 6))
			{
				setenv("TTS_VOICE", argv[argc] + 6, 1);
				setenv("ASR_VOICE", argv[argc] + 6, 1);
				continue;
			}

			// Added for domain vocabulary selection

			if(!strnicmp(argv[argc], "domain=", 7))
			{
				setenv("ASR_DOMAIN", argv[argc] + 7, 1);
				continue;
			}

			// Added for ASR event trapping

			if(!strnicmp(argv[argc], "asrevt=", 7))
			{
				setenv("ASR_EVENTS", argv[argc] + 7, 1);
				continue;
			}

			// Added for passing TTS phrases

			if(!strnicmp(argv[argc], "phrase=", 7))
			{
				while(NULL != (cp = strchr(argv[argc], '+')))
					*cp = ' ';
				while(NULL != (cp = strchr(argv[argc], ',')))
					*cp = ' ';
				setenv("TTS_PHRASE", argv[argc] + 7, 1);
				continue;
			}

			// Added for passing TTS source if file based

			if(!strnicmp(argv[argc], "source=", 7))
			{
				setenv("TTS_SOURCE", argv[argc] + 7, 1);
				continue;
			} 

			// Added for url helpers and such
			if(!strnicmp(argv[argc], "href=", 5))
			{
				setenv("URL_HREF", argv[argc] + 5, 1);
				continue;
			}

			if(!strnicmp(argv[argc], "base=", 5))
			{
				setenv("URL_BASE", argv[argc] + 5, 1);
				continue;
			}

			if(!strnicmp(argv[argc], "url=", 4))
			{
				setenv("URL_SOURCE", argv[argc] + 4, 1);
				continue;
			}

			// Added for passing TTS cache path

			if(!strnicmp(argv[argc], "cache=", 6))
			{
				setenv("TTS_CACHE", argv[argc] + 6, 1);
				continue;
			}

			++argc;
		}
		argv[argc] = NULL;
//		slog.close();
		close(0);
		close(1);
//		close(2);
		open("/dev/null", O_RDWR);
		dup2(cfd, 1);
//		dup2(cfd, 2);
		close(cfd);

		sprintf(buffer, "%s/%s", getenv("SERVER_LIBEXEC"), *argv);
		getInterp(buffer, argv);
		slog(SLOG_DEBUG) << "tgi: exec " << buffer << endl;
		execvp(buffer, argv);
		slog(SLOG_ERROR) << "tgi: exec failed; " << buffer << endl;
		exit(-1);
	}
	exit(SIGINT);
}

static void tgisignal(void)
{
	int count = keythreads.getGateways();
	int pid;

	if(!tgipids)
		return;

	for(pid = 0; pid < count; ++pid)
		kill(tgipids[pid], SIGUSR1);
}
		
static RETSIGTYPE final(int sig)
{
	int count = keythreads.getGateways();
	int i;

	if(getpid() != mainpid)
	{
		kill(mainpid, sig);
		ccxx_sleep(~0);
	}
	signal(SIGINT, SIG_IGN);
	signal(SIGABRT, SIG_IGN);
	signal(SIGTERM, SIG_IGN);
	signal(SIGQUIT, SIG_IGN);

	network.Stop();
	scheduler.Stop();
	stopServers();
	endMaps();

	if(resolver)
		delete resolver;

	if(driver)
		driver->Stop();

	if(sig)
		slog(SLOG_WARNING) << "exiting: reason=" << sig << endl;
	else
		slog(SLOG_NOTICE) << "normal shutdown" << endl;

	if(tgipids)
	{
		for(i = 0; i < count; ++i)
			kill(tgipids[i], SIGINT);
		tgipids = NULL;
	}

	if(restart_server)
		execvp(*restart_argv, restart_argv);
	exit(sig);
}

static void initial(int argc, char **argv)
{
	static struct option long_options[] = {
		{"background", 0, 0, 'D'},
		{"foreground", 0, 0, 'F'},
		{"daemon", 0, 0, 'D'},
		{"help", 0, 0, 'h'},
		{"priority", 1, 0, 'p'},
		{"port", 1, 0, 'P'},
		{"driver", 1, 0, 'd'},
		{"node", 1, 0, 'n'},
		{"test", 0, 0, 't'},
		{"trace", 0, 0, 'T'},
		{"thread", 0, 0, 'A'},
		{"version", 0, 0, 'V'},
		{"gui", 0, 0, 'G'},
		{"display", 1, 0, 'X'},
		{"aliases", 0, 0, 'a'},
		{"secure", 0, 0, 'S'},
		{0, 0, 0, 0}};

	static bool daemon = false;
	static bool usage = false;
	static bool gui = false;
	static bool pcount = 0;
	static bool mcount = 0;
	static bool su = false;

	struct utsname uts;
	TrunkGroup *policy;
	restart_argv = argv;
	char *envDriver = getenv("BAYONNE_DRIVER");
	Mixer *mixer;
	fstream mix;
	char prefix[256];
	const char *cp;
	char *pp, *tok;
	sigset_t sigs;
	int opt, opt_index;
	char *opt_script = NULL;
	int ports;
	bool test = false;
	bool schedule = true;
	char *etc = getenv("CONFIG_KEYDATA");
	struct stat ino;
	int count;
	ofstream php;
	const char *embed;

	cmds = argv;
	mainpid = getpid();
	sigemptyset(&sigs);
	sigaddset(&sigs, SIGTERM);
	sigaddset(&sigs, SIGQUIT);
	sigaddset(&sigs, SIGINT);
	pthread_sigmask(SIG_BLOCK, &sigs, NULL);
	slog.level(SLOG_NOTICE);

	uname(&uts);

	if(chkHosting(keyserver.getLast("hosting")))
	{
		if(!etc)
		{
			slog(SLOG_CRITICAL) << "bayonne: must use hosted config directory" << endl;
			final(-1);
		}
		ino.st_uid = 0xff;
		stat(etc, &ino);
		if(ino.st_uid)		
		{
			slog(SLOG_CRITICAL) << "bayonne: non-root owned host config directory" << endl;	
			final(-1);
		}
		ino.st_uid = 0xff;
		sprintf(prefix, "%s/hosting.conf", etc);
		stat(prefix, &ino);
		if(ino.st_uid)
		{
			slog(SLOG_CRITICAL) << "bayonne: non-root owned or missing hosting.conf" << endl;
			final(-1);
		}
	}

	if(envDriver)
		plugins.setValue("driver", envDriver);

	while(EOF != (opt = getopt_long(argc, argv, "P:GTDFX:hd:p:d:tn:aAS", long_options, &opt_index)))
		switch(opt)
		{
		case 'S':
			su = true;
			break;
		case 'X':
			setenv("DISPLAY", optarg, 1);
		case 'G':
			gui = true;
			plugins.setValue("debug", "gui");
			break;		
		case 'P':
			plugins.setValue("debug", "tcpmon");
			setenv("TCPMON", optarg, 1);
			break;	
		case 'V':
			cout << VERPATH << endl;
			exit(0);
		case 'A':
			keythreads.setValue("audit", "1");
			slog.level(SLOG_DEBUG);
			break;
		case 'a':
			aliases = true;
			break;
		case 'T':
			daemon = false;
			slog.level(SLOG_DEBUG);
			plugins.setValue("debug", "trace");
			break;			
		case 't':
			schedule = false;
			test = true;
			daemon = false;
			slog.level(SLOG_DEBUG);
			getcwd(prefix, sizeof(prefix) - 1);
			pp = strrchr(prefix, '/');
#if defined(have_montecarlo_h) || defined(HAVE_MONTECARLO_H)
			strcpy(pp, "/drivers/pika/pika.ivr");
#elif defined(HAVE_DIALOGIC_SDK)
			strcpy(pp, "/drivers/dialogic/dialogic.ivr");
#elif defined(HAVE_VPBAPI_H)
			strcpy(pp, "/drivers/vpb/vpb.ivr");
#elif defined(HAVE_CAPI20_H)
			strcpy(pp, "/drivers/capi20/capi20.ivr");
#elif defined(HAVE_LINUX_TELEPHONY_H)
			strcpy(pp, "/drivers/phonedev/phonedev.ivr");
#else
			strcpy(pp, "/drivers/dummy/dummy.ivr");
#endif
			if(envDriver)
				sprintf(pp, "/drivers/%s/%s.ivr",
					envDriver, envDriver);
			plugins.setValue("driver", prefix);
			strcpy(pp, "/modules/functions/string.fun");
			plugins.setValue("functions", prefix);
			strcpy(pp, "/modules/auditing/cdrsend.aud");
			plugins.setValue("auditing", prefix);
			strcpy(pp, "/modules/translators/english.tts");
			plugins.setValue("languages", prefix);
#ifdef	COMMON_XML_PARSING
			strcpy(pp, "/modules/xml/bayonne.xml");
			plugins.setValue("xml", prefix);
#endif
//			strcpy(pp, "/modules/perl/perl.tgi");
//			plugins.setValue("tgi", prefix);
			strcpy(pp, "/modules/protocols/mwi.mod");
			plugins.setValue("modules", prefix);
			if(gui)
				strcpy(pp, "/modules/gui/gui.dbg");
			else
				strcpy(pp, "/server/test.dbg");
			plugins.setValue("debug", prefix);
			strcpy(pp, "/data/script");
			keypaths.setValue("scripts", prefix);
			strcpy(pp, "/data");
			keypaths.setValue("prompts", prefix);
			keypaths.setValue("libexec", prefix);
			break;
		case 'n':
			keyserver.setValue("node", optarg);
			break;
		case 'd':
			plugins.setValue("driver", optarg);
			break;
		case 'p':
			keythreads.setValue("priority", optarg);
			break;
		case 'D':
			daemon = true;
			break;
		case 'F':
			daemon = false;
			break;
		default:
		case 'h':
			usage = true;
		}

	if(optind < argc)
	{
		if(test && !gui)
		{
			strcpy(pp, "/modules/auditing/trace.dbg");
			plugins.setValue("debug", prefix);
		}
		keyserver.setValue("default", argv[optind++]);
		schedule = false;
	}

	if(usage || optind < argc)
	{
		clog << "use: bayonne [-options] [defscript]" << endl;
		exit(-1);
	}

	endKeydata();
	
	keyserver.setGid();

	if(!getuid())
	{
		umask(003);
		if(!isDir(keypaths.getRunfiles()))
			mkdir(keypaths.getRunfiles(), 0770);
	}
	mkdir(keypaths.getDatafiles(), 0750);
	chdir(keypaths.getDatafiles());
	purge("temp");
	purge("cache");

	mkdir("temp", 0770);
	mkdir("prompts", 0770);
	mkdir("cache", 0770);
	mkdir("maps", 0770);
	mkdir("save", 0770);

	for(count = 0; count < 100; ++count)
	{
		sprintf(prefix, "cache/%02d", count);
		mkdir(prefix, 0770);
	}

	if(daemon)
	{
		close(0);
		close(1);
		close(2);
		pdetach();
		open("/dev/null", O_RDWR);
		open("/dev/null", O_RDWR);
		open("/dev/null", O_RDWR);
		mainpid = getpid();
		slog.open("bayonne", SLOG_DAEMON);
		slog(SLOG_NOTICE) << "daemon mode started" << endl;
	}
	else if(getppid() == 1)
	{
		close(0);
		close(1);
		close(2);
		open("/dev/null", O_RDWR);
		open("/dev/null", O_RDWR);
		open("/dev/null", O_RDWR);
		slog.open("bayonne", SLOG_DAEMON);
		slog(SLOG_NOTICE) << "daemon init started" << endl;
	}

	sprintf(prefix, "%s/bayonne", keypaths.getLibexec());
	setenv("SERVER_PLATFORM", plugins.getDriverName(), 1);
	setenv("SERVER_LIBEXEC", prefix, 1);
	setenv("SERVER_SOFTWARE", "bayonne", 1);
	setenv("SERVER_PROTOCOL", "2.2", 1);
	setenv("SERVER_VERSION", VERPATH, 1);
	setenv("SERVER_TOKEN", keyserver.getToken(), 1);

	strcpy(prefix, keyserver.getPrefix());
	strcat(prefix, "/bayonne");
	if(isDir(prefix))
	{
		strcat(prefix, ":");
		strcat(prefix, keypaths.getTgipath());
		setenv("PATH", prefix, 1);
	}
	else
		setenv("PATH", keypaths.getTgipath(), 1);

	slog(SLOG_INFO) << "SERVER VERSION " << VERPATH << "; ";
	slog() << uts.machine << " ";
	slog() << uts.sysname << " " << uts.release << endl;
	slog(SLOG_INFO) << "TGI VERSION 2.2";
	slog() << "; driver=" << plugins.getDriverName();
	slog() << "; prefix=" << prefix << endl;

	slog(SLOG_DEBUG) << "Loading TGI plugins..." << endl;
	try
	{
		plugins.loadTGI();
	}
	catch(DSO *dso)
	{
		slog(SLOG_CRITICAL) << dso->getError() << endl;
		final(-1);
	}
	tgi();
	sleep(1);
	rt();

	if(su)
		keyserver.setUid();

	try
	{
		slog(SLOG_DEBUG) << "Loading DSO plugin images..." << endl;
		plugins.loadManagers();
		if(!test)
			plugins.loadExtensions();
		plugins.loadDebug();
		plugins.loadDriver();
		plugins.loadFeeds();
		plugins.loadModules();
		plugins.loadServices();
		plugins.loadPreload();
		plugins.loadTranslators();
		plugins.loadAuditing();
		plugins.loadXML();
		driver->getImage();
	}
	catch(Driver *drv)
	{
		slog(SLOG_CRITICAL) << "multiple drivers loaded or driver failure" << endl;
		final(-1);
	}
	catch(Socket *sock)
	{
		slog(SLOG_CRITICAL) << "socket interface binding failure" << endl;
		final(-1);
	}
	catch(Dir *dir)
	{
		slog(SLOG_CRITICAL) << keypaths.getScriptFiles(); 
		slog() << ": no script directory" << endl;
		final(-1);
	}
	catch(DSO *dso)
	{
		slog(SLOG_CRITICAL) << dso->getError() << endl;
		final(-1);
	}
	catch(InetAddress *in)
	{
		slog(SLOG_CRITICAL) << "protocol resolver failed" << endl;
		final(-1);
	}
	catch(Thread *)
	{
		slog(SLOG_CRITICAL) << "service failure" << endl;
		final(-1);
	}
	catch(...)
	{
		slog(SLOG_CRITICAL) << "unknown failure" << endl;
		final(-1);
	}

	if(!driver)
	{
		slog(SLOG_CRITICAL) << "no driver loaded" << endl;
		final(-1);
	}

	keyserver.loadGroups(test);
	
	if(etc)
		strcpy(prefix, etc);
	else
		strcpy(prefix, ETC_PREFIX);
	strcat(prefix, "bayonne.clid");
	if(canAccess(prefix))
		new Map(prefix, "clid");
	else
	{
		strcpy(prefix, keyserver.getPrefix());
		strcat(prefix, "/clid.conf");
		if(canAccess(prefix))
			new Map(prefix, "clid");
	}

	if(etc)
		strcpy(prefix, etc);
	else
		strcpy(prefix, ETC_PREFIX);

	strcat(prefix, "bayonne.dnis");
	if(canAccess(prefix))
		new Map(prefix, "dnis");
	else
	{
		strcpy(prefix, keyserver.getPrefix());
		strcat(prefix, "/dnis.conf");
		if(canAccess(prefix))
			new Map(prefix, "dnis");
	}

	plugins.loadMaps();
	sync();
	ports = driver->Start();
	if(ports)
		slog(SLOG_NOTICE) << "driver started " << ports << " port(s)" << endl;
	else
	{
		slog(SLOG_CRITICAL) << "no trunk ports activated" << endl;
		final(-1);
	}
	policy = getGroup(NULL);
	embed = keypaths.getLast("embed");
	if(!embed)
		embed = "php";
	if(!*embed)
		embed = NULL;

        cp = keypaths.getLast("php");
        if(cp && !test)
               	php.open(cp, ios::out, 0660);
        if(php.is_open())
        {
		if(embed)
	               	php << "<?" << embed << endl;

		php << "# This file is automatically generated, do not edit" << endl;

               	php << "$BAYONNE_DRIVER=\"" 
		    << plugins.getDriverName() <<"\";" << endl;
		php << "$BAYONNE_HOME=\""
		    << getenv("HOME") << "\";" << endl;
		php << "$BAYONNE_LIBPATH=\""
		    << keypaths.getLibpath() << "\";" << endl;
		php << "$BAYONNE_LIBEXEC=\"" 
		    << getenv("SERVER_LIBEXEC") << "\";" << endl;
		php << "$BAYONNE_NODE=\""
		    << keyserver.getNode() << "\";" << endl;
		php << "$BAYONNE_TOKEN=\""
		    << keyserver.getToken() << "\";" << endl;
		php << "$BAYONNE_DATAFILES=\""
		    << keypaths.getDatafiles() << "\";" << endl;
		php << "$BAYONNE_RUNFILES=\""
		    << keypaths.getRunfiles() << "\";" << endl;
		php << "$BAYONNE_PROMPTS=\""
		    << keypaths.getPromptFiles() << "\";" << endl;
		php << "$BAYONNE_SCRIPTS=\""
		    << keypaths.getScriptFiles() << "\";" << endl;
		strcpy(prefix, policy->getLast("groups"));
		pp = strtok_r(prefix, " \t\n,;", &tok);
		while(pp)
		{
			php << "$BAYONNE_POLICY[" << pcount << "]=\"";
	  		php << pp << "\";" << endl;
			++pcount;
			pp = strtok_r(NULL, " \t\n,;", &tok);
		}
		php << "$BAYONNE_POLICIES="
		    << pcount << ";" << endl;
		php << "$BAYONNE_PORTS="
		    << ports << ";" << endl;
               	php << "$BAYONNE_VERSION=\"" VERPATH "\";" << endl;
		php << "$BAYONNE_MIXERS="
		    << driver->getMixers() << ";" << endl;
		while(mcount < driver->getMixers())
		{
			mixer = driver->getMixer(mcount);
			php << "$MIXER_GROUPS[" << mcount << "]=";
                        php << mixer->getGroups() << ";" << endl;
			php << "$MIXER_MEMBERS[" << mcount << "]=";
			php << mixer->getMembers() << ";" << endl;
			++mcount;
		}

		if(embed)
	               	php << "?>" << endl;
               	php.close();
		chown(cp, keyserver.getUid(), keyserver.getGid());
	}


	if(keythreads.getResolver())
		resolver = new Resolver();

	startServers();

	if(schedule)
		scheduler.Start();

	network.Start();

	if(canModify(keypaths.getRunfiles()))
	{
		strcpy(prefix, keypaths.getRunfiles());
		strcat(prefix, "/bayonne.mixer");
	}
	else
		sprintf(prefix, "%s/.bayonne.mixer", getenv("HOME"));

	remove(prefix);
	mix.open(prefix, ios::out);
	if(mix.is_open())
	{
		chown(prefix, keyserver.getUid(), keyserver.getGid());
		count = 0;
		while(count < driver->getMixers())
		{
			mixer = driver->getMixer(count);
			if(!mixer)
				break;

			mix << count++ << " " << mixer->getGroups() << " " << mixer->getMembers() << endl;
		}
		mix.close();
	}
	else
		slog(SLOG_WARNING) << "startup: mixer open failed" << endl;

	signal(SIGTERM, final);
	signal(SIGQUIT, final);
	signal(SIGINT, final);
	signal(SIGABRT, final);
	pthread_sigmask(SIG_UNBLOCK, &sigs, NULL);
	slog("bayonne", SLOG_DAEMON);
	if(keyserver.getLast("config"))
		slog(SLOG_NOTICE) << "normal startup; " << keyserver.getLast("config") << endl;
	else
		slog(SLOG_NOTICE) << "normal startup" << endl;

	running = true;
}

int main(int argc, char **argv)
{
	static char buffer[256];
	char *p;
	int i;

	initial(argc, argv);

	if(debug->DebugTest())
		final(0);

	if(canModify(keypaths.getRunfiles()))
	{
		strcpy(buffer, keypaths.getRunfiles());
		strcat(buffer, "/bayonne.ctrl");
	}
	else
		sprintf(buffer, "%s/.bayonne.ctrl", getenv("HOME"));

	fifo.open(buffer);
	slog(SLOG_DEBUG) << "fifo: path=" << buffer << endl;
	if(!fifo.is_open())
	{
		slog(SLOG_WARNING) << "fifo: open failed" << endl;
		sleep(-1);
	}

	chown(buffer, keyserver.getUid(), keyserver.getGid());

	for(;;)
	{
		fifo.getline(buffer, sizeof(buffer) - 1);
		if(fifo.eof())
		{
			slog(SLOG_CRITICAL) << "fifo failure" << endl;
			final(-1);
		}
		p = buffer + strlen(buffer) - 1;
		while(p > buffer)
		{
			if(*p == ' ' || *p == '\t' || *p == '\n' || *p == '\r')
				*(p--) = 0;
			else
				break;
		}
		p = buffer;
		while(*p == ' ' || *p == '\t')
			++p;
		if(!*p)
			continue;
		fifo.Command(p);
	}
	fifo.close();
	final(0);
}

