// Copyright (C) 2000-2001 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "bayonne.h"
#include <iomanip>

class TestDebug : public Debug
{
private:
	bool script;
	int test;
	int success;

public:
	TestDebug();
	void DebugService(Trunk *trunk, char *msg);
	void DebugState(Trunk *trunk, char *state);
	void DebugScript(Trunk *trunk, char *msg);
	void DebugStep(Trunk *trunk, scriptline_t *line);
	bool DebugTest(void);
	bool DebugFifo(char **argv);
} testdebug;

TestDebug::TestDebug() :
Debug()
{
	success = 0;
	test = 0;
	slog(SLOG_INFO) << "debug: test module loaded" << endl;
}

bool TestDebug::DebugFifo(char **argv)
{
	++success;
	slog(SLOG_INFO) << "debug: fifo->";
	while(*argv)
		slog() << ' ' << *(argv++);
	slog() << endl;
	return true;
}

bool TestDebug::DebugTest(void)
{
	TrunkEvent event;
	Trunk *trunk = driver->getTrunkPort(0);

	if(!trunk)
	{
		slog(SLOG_ERROR) << "debug: trunk 0 not available for tests" << endl;
		return true;
	}
	rmdir("temp");
	rmdir("cache");
	rmdir("maps");
	rmdir("prompts");
	trunk->group->setValue("answer", "1");
	trunk->group->setValue("xml", "bayonnexml");
	slog(SLOG_INFO) << "debug: *** start of regression test series" << endl;
	ccxx_sleep(1000);
	event.id = TRUNK_MAKE_IDLE;
	trunk->postEvent(&event);
	ccxx_sleep(1000);

	slog(SLOG_INFO) << "debug: *** start of cdr test" << endl;
	++test;
	script = false;
	keyserver.setValue("default", "test::test1");
	trunk->group->setValue("name", "test::test1");
	event.parm.ring.digit = 0;
	event.id = TRUNK_RINGING_ON;
	trunk->postEvent(&event);
	ccxx_sleep(250);
	event.id = TRUNK_RINGING_OFF;
	trunk->postEvent(&event);
	ccxx_sleep(2000);
	if(script)
	{
		++success;
		slog(SLOG_INFO) << "debug: *** test cdr successful" << endl;
	}
	else
		slog(SLOG_INFO) << "debug: *** test cdr failed" << endl;

	slog(SLOG_INFO) << "debug: *** start of state handler test" << endl;
	++test;
	script = false;
	keyserver.setValue("default", "test::test2");
	trunk->group->setValue("name", "test::test2");
	event.parm.ring.digit = 0;
	event.id = TRUNK_RINGING_ON;
	trunk->postEvent(&event);
	ccxx_sleep(250);
	event.id = TRUNK_RINGING_OFF;
	trunk->postEvent(&event);
	ccxx_sleep(4000);
	if(script)
	{
		++success;
		slog(SLOG_INFO) << "debug: *** test state successful" << endl;
	}
	else
		slog(SLOG_INFO) << "debug: *** test state failed" << endl;

	slog(SLOG_INFO) << "debug: *** start of trap test" << endl;
	++test;
	script = false;
	keyserver.setValue("default", "test::test3");
	trunk->group->setValue("name", "test::test3");
	event.parm.ring.digit = 0;
	event.id = TRUNK_RINGING_ON;
	trunk->postEvent(&event);
	event.id = TRUNK_RINGING_OFF;
	trunk->postEvent(&event);
	ccxx_sleep(500);
	event.id = TRUNK_DTMF_KEYUP;
	event.parm.dtmf.digit = 1;
	trunk->postEvent(&event);
	ccxx_sleep(1000);
	if(script)
	{
		++success;
		slog(SLOG_INFO) << "debug: *** test trap successful" << endl;
	}
	else
		slog(SLOG_INFO) << "debug: *** test trap failed" << endl;

#ifdef	COMMON_XML_PARSING
	slog(SLOG_INFO) << "debug: *** start of test xml" << endl;
	++test;
	script = false;
	keyserver.setValue("default", "test::test4");
	trunk->group->setValue("name", "test::test4");
	event.parm.ring.digit = 0;
	event.id = TRUNK_RINGING_ON;
	trunk->postEvent(&event);
	event.id = TRUNK_RINGING_OFF;
	trunk->postEvent(&event);
	ccxx_sleep(2000);
	if(script)
	{
		++success;
		slog(SLOG_INFO) << "debug: *** test xml successful" << endl;
	}
	else
		slog(SLOG_INFO) << "debug: *** test xml failed" << endl;
#endif

	ccxx_sleep(2000);
	++test;
	slog(SLOG_INFO) << "debug: *** fifo test..." << endl;
	fifo.Command("debug&first&last");
	slog(SLOG_INFO) << "debug: *** test series completed ";
	slog() << success << " of " << test << " successful" << endl;
	return true;
}

void TestDebug::DebugService(Trunk *trunk, char *msg)
{
	char buffer[32];
	if(test > 1)
		return;

	EnterMutex();
	if(trunk)
	{
		trunk->getName(buffer);
		slog(SLOG_DEBUG) << "debug: " << buffer << ": ";
	}
	else
		slog(SLOG_DEBUG) << "debug: ";
	slog() << "service " << msg << endl;
	LeaveMutex();
}

void TestDebug::DebugScript(Trunk *trunk, char *msg)
{
	char buffer[32];
	char *value;

	script = true;
	EnterMutex();
	script = true;
	trunk->getName(buffer);
	slog(SLOG_DEBUG) << buffer << ": " << msg << endl;
	LeaveMutex();
}

void TestDebug::DebugStep(Trunk *trunk, scriptline_t *line)
{
	int i;

	char buffer[32];
	trunk->getName(buffer);

	if(!line)
	{
		slog(SLOG_DEBUG) << buffer << ": exit" << endl;
		return;
	}

	EnterMutex();
	slog(SLOG_DEBUG) << buffer << ": step ";
	slog() << setbase(16) << line->mask << " " << setbase(10);
	slog() << line->cmd << "(";
	for(i = 0; i < line->argc; ++i)
	{
		if(i)
			slog() << ",";
		slog() << line->args[i];
	}
	slog() << ")" << endl;	
	LeaveMutex();
}

void TestDebug::DebugState(Trunk *trunk, char *state)
{
	char buffer[32];
	trunk->getName(buffer);

	EnterMutex();
	slog(SLOG_DEBUG) << buffer << ": " << state << endl;
	LeaveMutex();
}

